# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["PaymentStatus"]

PaymentStatus: TypeAlias = Literal["pending", "requires_intervention", "paid", "canceled"]
